@@ -1,18 +1,20 @@
 import logging
+import re
 from functools import partial
 from pathlib import Path
 from typing import Dict, List, Tuple
 
+from bridgebots.bids import canonicalize_bid
+from bridgebots.board_record import BidMetadata, BoardRecord
 from bridgebots.deal import Card, Deal
 from bridgebots.deal_enums import BiddingSuit, Direction, Suit
 from bridgebots.deal_utils import from_pbn_deal
-from bridgebots.table_record import TableRecord
 
 
 def _split_pbn(file_path: Path) -> List[List[str]]:
     """
-    Read in the entire pbn file. Split on lines consisting of '*\n' into a list of strings per board
-    :param file_path: path to pbn file
+    Read in the entire PBN file. Split on lines consisting of '*\n' into a list of strings per board
+    :param file_path: path to PBN file
     :return:
     """
     with open(file_path, "r") as pbn_file:
@@ -20,9 +22,9 @@ def _split_pbn(file_path: Path) -> List[List[str]]:
         current_record = ""
         while True:
             line = pbn_file.readline()
-            if line == "":
+            if line == "":  # EOF
                 return records
-            elif line == "*\n":
+            elif line == "\n":  # End of Board Record
                 records.append(current_record.splitlines())
                 current_record = ""
             else:
@@ -42,16 +44,31 @@ def _build_record_dict(record_strings: List[str]) -> Dict:
     i = 0
     while i < len(record_strings):
         record_string = record_strings[i]
-        if not record_string.startswith("["):
+        if not (record_string.startswith("[") or record_string.startswith("{")):
             i += 1
             continue
-        if "[" in record_string and "]" not in record_string:
+        if record_string.startswith("{"):
+            commentary = ""
+            while i < len(record_strings):
+                record_string = record_strings[i]
+                if record_string.startswith("["):
+                    break
+                commentary += record_string + " "
+                i += 1
+            record_dict["Commentary"] = commentary.strip()
+            continue
+        if record_string.startswith("[") and "]" not in record_string:
             while "]" not in record_string:
                 i += 1
                 record_string = record_string + record_strings[i]
         record_string = record_string.replace("[", "").replace("]", "")
         key, value = record_string.split(maxsplit=1)
-        record_dict[key] = value.replace('"', "")
+        value = value.replace('"', "")
+        if key == "Note":
+            number, message = value.split(":", maxsplit=1)
+            key = key + "_" + number
+            value = message
+        record_dict[key] = value
         if key == "Auction":
             auction_record = []
             i += 1
@@ -61,9 +78,6 @@ def _build_record_dict(record_strings: List[str]) -> Dict:
                     break
                 auction_record.extend(auction_str.split())
                 i += 1
-            if auction_record and auction_record[-1] in ["AP", "ap"]:
-                auction_record = auction_record[:-1]  # Replace 'AP' with 3 passes
-                auction_record.extend(["Pass"] * 3)
             record_dict["bidding_record"] = auction_record
 
         elif key == "Play":
@@ -71,7 +85,7 @@ def _build_record_dict(record_strings: List[str]) -> Dict:
             i += 1
             while i < len(record_strings):
                 play_str = record_strings[i]
-                if "[" in play_str:
+                if "[" in play_str or play_str == "*":
                     break
                 play_record.append(play_str.split())
                 i += 1
@@ -81,6 +95,67 @@ def _build_record_dict(record_strings: List[str]) -> Dict:
     return record_dict
 
 
+def _update_bidding_metadata(
+    bid_index: int, raw_bid: str, bidding_record: List[str], bidding_metadata: List[BidMetadata], record_dict: dict
+):
+    """
+    Create or update bidding metadata for the most recent bid
+    """
+    if len(bidding_record) == 0:
+        return
+    if bidding_metadata and bidding_metadata[-1].bid_index == bid_index - 1:
+        bid_metadata = bidding_metadata[-1]
+    else:
+        bid_metadata = BidMetadata(bid_index=bid_index - 1, bid=bidding_record[-1])
+        bidding_metadata.append(bid_metadata)
+    if raw_bid == "!":
+        bid_metadata.alerted = True
+    else:
+        # Attempt to determine which note should be used as an explanation
+        match = re.match("=([0-9]+)=", raw_bid)
+        if match:
+            note_key = "Note_" + match.group(1)
+            explanation = record_dict.get(note_key) if note_key in record_dict else raw_bid
+        else:
+            explanation = raw_bid
+        if bid_metadata.explanation:
+            bid_metadata.explanation += " | " + explanation
+        else:
+            bid_metadata.explanation = explanation
+
+
+def _parse_bidding_record(raw_bidding_record: List[str], record_dict: dict) -> Tuple[List[str], List[BidMetadata]]:
+    """
+    Check each auction string. If it is a valid bid, add it to the bidding record. If not, create/update BidMetadata for
+    the previous bid. Example auction:
+
+    1C 2NT =0= ! 3H =1= pass
+    3S pass 3NT pass
+    pass pass
+
+    For 2NT create a BidMetaData which looks up Note_0 from record_dict as an explanation and sets the alert flag to
+    true
+
+    :param raw_bidding_record: All strings from the auction section of the PBN
+    :param record_dict: Other fields from the PBN, including notes (bid explanations)
+    :return: cleaned bidding record and bidding metadata
+    """
+    bid_index = 0
+    bidding_record = []
+    bidding_metadata = []
+    for raw_bid in raw_bidding_record:
+        canonical_bid = canonicalize_bid(raw_bid)
+        if canonical_bid:
+            bidding_record.append(canonical_bid)
+            bid_index += 1
+        elif raw_bid.upper() == "AP":
+            bidding_record.extend(["PASS"] * 3)
+            bid_index += 3
+        else:
+            _update_bidding_metadata(bid_index, raw_bid, bidding_record, bidding_metadata, record_dict)
+    return bidding_record, bidding_metadata
+
+
 def _evaluate_card(trump_suit: Suit, suit_led: Suit, card: Card) -> int:
     """
     :return: Score a card on its ability to win a trick given the trump suit and the suit that was led to the trick
@@ -116,7 +191,7 @@ def _sort_play_record(trick_records: List[List[str]], contract: str) -> List[Car
     if contract == "":
         logging.warning(f"empty contract, cannot determine play ordering: {trick_records}")
         return []
-    if contract == "Pass":
+    if contract.upper() == "PASS":
         return []
     try:
         trump_suit = BiddingSuit.from_str(contract[1:2]).to_suit()
@@ -143,22 +218,24 @@ def _sort_play_record(trick_records: List[List[str]], contract: str) -> List[Car
         return []
 
 
-def _parse_table_record(record_dict: Dict) -> TableRecord:
+def _parse_board_record(record_dict: Dict) -> BoardRecord:
     """
-    Convert the record dictionary to a TableRecord
+    Convert the record dictionary to a BoardRecord
     :param record_dict: mapping of PBN keys to deal or board information
-    :return: TableRecord representing the people and actions at the table
+    :return: BoardRecord representing the people and actions at the table
     """
     declarer_str = record_dict["Declarer"]
     declarer = Direction.from_str(declarer_str) if declarer_str and declarer_str != "" else None
-    bidding_record = record_dict.get("bidding_record") or []
+    raw_bidding_record = record_dict.get("bidding_record") or []
+    bidding_record, bidding_metadata = _parse_bidding_record(raw_bidding_record, record_dict)
     play_record_strings = record_dict.get("play_record") or []
     play_record = _sort_play_record(play_record_strings, record_dict["Contract"])
     result_str = record_dict.get("Result")
     result = int(result_str) if result_str and result_str != "" else None
 
-    return TableRecord(
+    return BoardRecord(
         bidding_record=bidding_record,
+        raw_bidding_record=raw_bidding_record,
         play_record=play_record,
         declarer=declarer,
         contract=record_dict["Contract"],
@@ -170,20 +247,35 @@ def _parse_table_record(record_dict: Dict) -> TableRecord:
         west=record_dict.get("West"),
         date=record_dict.get("Date"),
         event=record_dict.get("Event"),
+        bidding_metadata=bidding_metadata,
+        commentary=record_dict.get("Commentary"),
     )
 
 
-def parse_pbn(file_path: Path) -> List[Tuple[Deal, TableRecord]]:
+def _parse_single_pbn_record(record_strings: List[str]) -> Tuple[Deal, BoardRecord]:
     """
-    Split pbn file into boards then decompose those boards into Deal and TableRecord objects
-    :param file_path: path to a pbn file
-    :return: A list of pairs of Deal and TableRecord
+    :param record_strings: One string per line of a single PBN deal record
+    :return: Deal and BoardRecord corresponding to the PBN record
+    """
+    record_dict = _build_record_dict(record_strings)
+    deal = from_pbn_deal(record_dict["Dealer"], record_dict["Vulnerable"], record_dict["Deal"])
+    board_record = _parse_board_record(record_dict)
+    return deal, board_record
+
+
+def parse_pbn(file_path: Path) -> List[Tuple[Deal, BoardRecord]]:
+    """
+    Split PBN file into boards then decompose those boards into Deal and BoardRecord objects. Only supports PBN v1.0
+    See https://www.tistis.nl/pbn/pbn_v10.txt
+
+    :param file_path: path to a PBN file
+    :return: A list of pairs of Deal and BoardRecord
     """
     records_strings = _split_pbn(file_path)
     results = []
     for record_strings in records_strings:
-        record_dict = _build_record_dict(record_strings)
-        deal = from_pbn_deal(record_dict["Dealer"], record_dict["Vulnerable"], record_dict["Deal"])
-        table_record = _parse_table_record(record_dict)
-        results.append((deal, table_record))
+        try:
+            results.append(_parse_single_pbn_record(record_strings))
+        except KeyError as e:
+            logging.warning(f"Malformed record {record_strings}: {e}")
     return results
